
--
--
-- Adjusting background quality is now available from in-game.
-- Zmiana właściwości tła jest teraz dostępna w trakcie gry.
--
--

--tilesize = 32
horizon = 288

poleStarX = 960
poleStarY = -320

local A1, A2 = 727595, 798405  -- 5^17=D20*A1+A2
local D20, D40 = 1048576, 1099511627776  -- 2^20, 2^40
local X1, X2 = 0, 1
function rand()
    local U = X2*A2
    local V = (X1*A2 + X2*A1) % D20
    V = (V*D20 + U) % D40
    X1 = math.floor(V/D20)
    X2 = V - X1*D20
    return V/D40
end

function randomizeBrightness()
	local a = rand()
	if a < 0.05 then
		return 4
	elseif a < 0.15 then
		return 3
	elseif a < 0.35 then
		return 2
	else
		return 1
	end
end

function getDistance(x1, x2, y1, y2)
	return ((x1-x2)^2+(y1-y2)^2)^0.5
end


function returnFrame(a)
	-- 0 - bright
	-- 1 - dark
	local b=1
	if a<0 then
		b=0
	elseif a>1 then
		b=1
	else
		b=a
	end
	return b*20+1
end

function calcBrightness(self, tileX, tileY, sunX, sunY)
	PLAY_AREA = ffi.cast("Rect*", 0x535840)[0]
	local Horizon = (PLAY_AREA.Bottom + 1) * 0.6
	return getDistance(tileX, sunX, tileY, sunY)/(700) - ( math.min(tileY, Horizon+self.tilesize)/math.max(tileY, Horizon+self.tilesize) )^3
end

function isWithinPlayArea(x, y, rightEdge, botEdge)
	if x<0 or x>rightEdge or y<0 or y>botEdge then
		return false
	else
		return true
	end
end

function getPerceptiveStarBrightness(bgDarkness, starBrightness)
	--TextOut(bgDarkness)
	local result = bgDarkness+starBrightness*4
	if result >= 36 then
		return 4
	end
	if result >= 32 then
		return 3
	end
	if result >= 28 then
		return 1
	end
	if result >= 24 then
		return 0
	end
	return -1
	
end

function ClearBg(self)
	for i, obj in ipairs(self.background) do
		obj:Destroy()
	end
	--self.background = nil
	for i, obj in ipairs(self.backgroundWater) do
		obj:Destroy()
	end
	--self.backgroundWater = nil
end
function CreateBg(self)
	self.background = {}
	self.backgroundWater = {}
	
	self.sunPosX = 192
	self.sunPosY = 320
	local flags = ffi.new("Flags_t")
	flags.AlwaysActive = true
	PLAY_AREA = ffi.cast("Rect*", 0x535840)[0]
	local Horizon = (PLAY_AREA.Bottom + 1) * 0.6
	-- PLAY_AREA.Right ~= 639 and PLAY_AREA.Bottom ~= 479
	for i = self.tilesize/2,(PLAY_AREA.Right+1)-self.tilesize/2,self.tilesize do
		for j = self.tilesize/2,Horizon-self.tilesize/2,self.tilesize do
			local bgFrag = CreateObject {x=i, y=j, z=-9000, SpeedY=0, logic="BackgroundLogic", image=nightskyimage, Flags=flags}
			bgFrag.Counter = j
			--bgFrag.DrawFlags.NoDraw = true
			bgFrag:SetFrame(21)
			table.insert(self.background, bgFrag)
		end
		for k = Horizon-32,(PLAY_AREA.Bottom+1),64 do
			local bgFrag = CreateObject {x=i, y=k, z=-7000, logic="BackgroundLogic", image="CUSTOM_NIGHTWATER", Flags=flags}
			bgFrag:SetFrame(1)
			--bgFrag.DrawFlags.NoDraw = true
			table.insert(self.backgroundWater, bgFrag)
		end
	end
	
end

function StarsMovement(self, Twilight_Quality)
	local flags = ffi.new("Flags_t")
	flags.AlwaysActive = true
	for j, star in ipairs(self.stars) do
		star[4] = math.cos(math.rad(star[1]+self.angle))*star[2]+poleStarX
		star[5] = math.sin(math.rad(star[1]+self.angle))*star[2]+poleStarY
		local starBgBrightness = returnFrame( calcBrightness(self, star[4], star[5], self.sunPosX, self.sunPosY) )
		-- the more, the more visible, the darker
		local perceptiveStarBrightness = -1
		local perceptiveStarBrightness = getPerceptiveStarBrightness(starBgBrightness, star[6])
		PLAY_AREA = ffi.cast("Rect*", 0x535840)[0]
		-- PLAY_AREA.Right ~= 639 and PLAY_AREA.Bottom ~= 479
		local xFactor = ( PLAY_AREA.Right ) / 640
		local yFactor = ( PLAY_AREA.Bottom ) / 480
		if star[3] == nil then
			if isWithinPlayArea(star[4], star[5], 640, 480) and perceptiveStarBrightness>=0 then
				star[3] = CreateObject {x=star[4]*xFactor, y=star[5]*yFactor, z=-8000, logic="BackgroundLogic", image="GAME_SPARKLE2", Flags=flags}
				star[3].I = perceptiveStarBrightness
				star[3]:SetFrame(perceptiveStarBrightness)
				if Twilight_Quality == 0 then
					star[3].DrawFlags.NoDraw = true
				else
					star[3].DrawFlags.NoDraw = false
				end
			end
		else
			
			if isWithinPlayArea(star[4], star[5], 640, 480) and perceptiveStarBrightness>=0 then
				--TextOut(star[3])
				star[3].X = star[4]*xFactor
				star[3].Y = star[5]*yFactor
				star[3].I = perceptiveStarBrightness
				star[3]:SetFrame(perceptiveStarBrightness)
				if Twilight_Quality == 0 then
					star[3].DrawFlags.NoDraw = true
				else
					star[3].DrawFlags.NoDraw = false
				end
			else
				local obj = star[3]
				obj:Destroy()
				star[3] = nil
			end
		end
		
	end
end

function Redraw(self)
	
	PLAY_AREA = ffi.cast("Rect*", 0x535840)[0]
	-- PLAY_AREA.Right ~= 639 and PLAY_AREA.Bottom ~= 479
	
	
	if Twilight_Quality == 1 then
		self.tilesize = 64
		nightskyimage = "CUSTOM_NIGHTSKY"
	elseif Twilight_Quality == 2 then
		self.tilesize = 32
		nightskyimage = "CUSTOM_NIGHTSKYHD"
	else
		self.tilesize = 64
		nightskyimage = "CUSTOM_NIGHTSKY"
	end
	
	if Twilight_Quality == 0 then
		ClearBg(self)
	else
		ClearBg(self)
		CreateBg(self)
	end
	
end

function init(self)
	self.firstLaunch = 0
	self.file = io.open("Custom/Raid on the Spanish Armada/nightsky.txt", "r")
	if self.file == nil then -- save file does not exist
		self.file = io.open("Custom/Raid on the Spanish Armada/nightsky.txt", "w")
		io.output(self.file)
		io.write("2")
		io.close(self.file)
		self.firstLaunch = 1
		TextOut("First launch detected. We will auto-select background quality based on your PC performance.")
		Twilight_Quality = 2
	else -- save file exists
		local textdump = self.file:read()
		Twilight_Quality = tonumber(textdump)
		
		io.close(self.file)
	end
	
	
	
	Star_Count = 400
	if Twilight_Quality == 1 then
		self.tilesize = 64
		nightskyimage = "CUSTOM_NIGHTSKY"
	elseif Twilight_Quality == 2 then
		self.tilesize = 32
		nightskyimage = "CUSTOM_NIGHTSKYHD"
	else
		self.tilesize = 64
		nightskyimage = "CUSTOM_NIGHTSKY"
	end
	CreateBg(self)
	Redraw(self)
	
	self.angle = 0
	self.TimePenalty = {0}
	
	PLAY_AREA = ffi.cast("Rect*", 0x535840)[0]
	self.lastWidth = PLAY_AREA.Right
	self.lastHeight = PLAY_AREA.Bottom
	
	self.sunStartPosX = 0.3
	self.sunStartPosY = 2/3
	
	-- 640x480
	local minDist = getDistance(poleStarX, 640, poleStarY, 0)
	local maxDist = getDistance(poleStarX, 0, poleStarY, 480)
	local diffDist = maxDist - minDist
	
	self.stars = {} -- { angle, distance from poleStar, star object, x, y, star brightness }
	for i = 1, Star_Count, 1 do
		table.insert(self.stars, {rand()*360,rand()*diffDist+minDist, nil, 0, 0, randomizeBrightness()})
	end
	
	--CreateBg(self)

end

function main(self)
	if self.firstLaunch == 1 and GetTime() > 2000 then
		self.FirstLaunchFpsCount = nRes(6)
		self.firstLaunch = 2
	end
	if self.firstLaunch == 2 and GetTime() > 3000 then
		self.FirstLaunchFpsCount = self.FirstLaunchFpsCount + nRes(6)
		self.firstLaunch = 3
	end
	if self.firstLaunch == 3 and GetTime() > 4000 then
		self.FirstLaunchFpsCount = self.FirstLaunchFpsCount + nRes(6)
		self.firstLaunch = 4
	end
	if self.firstLaunch == 4 and GetTime() > 5000 then
		self.FirstLaunchFpsCount = self.FirstLaunchFpsCount + nRes(6)
		self.firstLaunch = 5
	end
	if self.firstLaunch == 5 and GetTime() > 6000 then
		self.FirstLaunchFpsCount = self.FirstLaunchFpsCount + nRes(6)
		self.firstLaunch = 6
		local avgFPS = self.FirstLaunchFpsCount / 5
		if avgFPS >= 70 then
			TextOut("Your framerate is above recommended 60 FPS.")
		elseif avgFPS >= 54 then
			TextOut("Your framerate is within recommended 60 FPS.")
		elseif avgFPS >= 32 then
			TextOut("Your framerate is lower than recommended 60 FPS. Reducing background quality.")
			Twilight_Quality = 1
			Redraw(self)
			self.file = io.open("Custom/Raid on the Spanish Armada/nightsky.txt", "w")
			io.output(self.file)
			io.write("1")
			io.close(self.file)
		else
			TextOut("Your framerate is lower than recommended 60 FPS. Disabling custom background.")
			Twilight_Quality = 0
			Redraw(self)
			self.file = io.open("Custom/Raid on the Spanish Armada/nightsky.txt", "w")
			io.output(self.file)
			io.write("0")
			io.close(self.file)
		end
	end
	if self.firstLaunch == 6 and GetTime() > 12000 then
		self.firstLaunch = 0
		TextOut("Regardless, you can change background quality using F2, F3, or F4 keys.")
	end
	
	-- 0x71 F2
	-- 0x72 F3
	-- 0x73 F4
	if KeyPressed(0x71) and Twilight_Quality~= 2 then
		Twilight_Quality = 2
		Redraw(self)
		self.file = io.open("Custom/Raid on the Spanish Armada/nightsky.txt", "w")
		io.output(self.file)
		io.write("2")
		io.close(self.file)
	end
	if KeyPressed(0x72) and Twilight_Quality~= 1 then
		Twilight_Quality = 1
		Redraw(self)
		self.file = io.open("Custom/Raid on the Spanish Armada/nightsky.txt", "w")
		io.output(self.file)
		io.write("1")
		io.close(self.file)
	end
	if KeyPressed(0x73) and Twilight_Quality~= 0 then
		Twilight_Quality = 0
		Redraw(self)
		self.file = io.open("Custom/Raid on the Spanish Armada/nightsky.txt", "w")
		io.output(self.file)
		io.write("0")
		io.close(self.file)
	end
	
	--if Twilight_Quality == 0 then
		--ClearBg(self)
	--end
	PLAY_AREA = ffi.cast("Rect*", 0x535840)[0]
	if PLAY_AREA.Right ~= self.lastWidth or PLAY_AREA.Bottom ~= self.lastHeight then
		self.lastWidth = PLAY_AREA.Right
		self.lastHeight = PLAY_AREA.Bottom
		Redraw(self)
	end
	
	--if PLAY_AREA.Right ~= 639 and PLAY_AREA.Bottom ~= 479 and Twilight_Quality ~= 0 then
	--	StarsMovement(self, Twilight_Quality)
	--	return 0
	--end
	
	self.angle = self.angle - 0.0001
	--self.angle = self.angle - 0.0001
	if self.angle >= 360 then
		self.angle = 0
	end
	--self.sunPosY = self.sunPosY + 0.4
	
	self.sunPosX = (PLAY_AREA.Right + 1)* self.sunStartPosX 
	local Achevements = GetObject(7250)
	
	self.sunPosY = (PLAY_AREA.Bottom + 1) * self.sunStartPosY + 0.0006 * ( GetTime()+self.TimePenalty[1] ) * (PLAY_AREA.Bottom + 1)/480
	if Twilight_Quality ~= 0 then
	
		for i, bgFrag in ipairs(self.background) do
			--bgFrag:SetFrame( 1 )
			bgFrag:SetFrame( returnFrame( calcBrightness(self, bgFrag.X, bgFrag.Counter, self.sunPosX, self.sunPosY) ) )
		end
		
		
	end
	
	StarsMovement(self, Twilight_Quality)
end

